/*
 * Copyright (c) 2010 ProfitBricks GmbH. All Rights Reserved.
 */
package org.glassfish.cditest.security.interceptor;

import java.lang.reflect.Method;
import java.security.Principal;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.annotation.Resource;
import javax.ejb.EJBContext;
import javax.interceptor.AroundInvoke;
import javax.interceptor.Interceptor;
import javax.interceptor.InvocationContext;

import java.io.Serializable;
import org.glassfish.cditest.security.api.Secure;

/**
 * Realizes security for EJBs.
 * 
 * @author ifischer
 * 
 */
@Secure
@Interceptor
public class SecurityInterceptor implements Serializable
{
    private static final Logger LOG = Logger.getLogger(SecurityInterceptor.class.getName());
    
    @Resource
    private EJBContext ejbCtx;
    
    /**
     * Perform lookup for permissions.
     * Does the caller has the permission to call the method?
     * TODO: implement lookup
     * 
     * @param InvocationContext of intercepted method
     * @return
     * @throws Exception
     */
    @AroundInvoke
    protected Object invoke(final InvocationContext ctx) throws Exception
    {
        Principal p = ejbCtx.getCallerPrincipal();
        Method interfaceMethod = ctx.getMethod();
        
        LOG.log(Level.INFO, "EJB Method called [Full]:\"{0}\" by Principal:{1}", new Object[]{getFullEJBClassName(interfaceMethod), p.toString()});
        LOG.log(Level.INFO, "EJB Method called [Methodonly]:{0} by Principal:{1}", new Object[]{interfaceMethod.getName(), p.toString()});
        
        return ctx.proceed();
    }
    
    /**
     * The EJBContext interface doesn't provide convenient methods to get the name of the EJB class, 
     * so the classname has to be extracted from the method.
     * 
     * @param the method whose classname is needed
     * @return classname (fully qualified) of given method, e.g. "com.profitbricks.user.api.UserService"
     */
    private String getFullEJBClassName(Method method) {
        // extract className from methodName
        // methodName format example:"public void com.profitbricks.user.api.UserService.testMe()"
        String methodName = method.toString();
        
        int start = methodName.lastIndexOf(' ') + 1;
        int end = methodName.lastIndexOf('.');
        
        String className = methodName.substring(start, end);
        
        return className;
    }
}